@@ -15,13 +15,13 @@ use POSIX;
 
 my $hostname = `hostname`;
 $hostname =~ s/\n//g;
-
 # configfile is the only thing you should set. Other stuff is overriden by
 # contents of the config file
 my $configfile = '/etc/viperdb.conf';
 my $dbfname = '.ViperDB';
 my $lock = "/var/lock/subsys/viperdb.lock";
 my $md5lock = "/var/lock/subsys/viperdb-md5.lock";
+my $tielimit = 5;
 my $strict = 0;
 my $init = 0;
 my $doattr = 0;
@@ -34,7 +34,7 @@ my %ignore;
 my @rignore;
 my $md5obj = new MD5;
 
-# Grab da OS name
+#Grab da OS name
 chomp($os_type=lc(qx[uname]));
 #open LOG, "|cat";
 
@@ -62,7 +62,7 @@ if ($ARGV[0] eq '-init'){
 	print "Check Detected: Now Checking File Sanity...\n";
 	&Go;
 } else {
-	print "\n\nViperDB v0.9.6\n";
+	print "\n\nViperDB v0.9.7\n";
 	print "ERROR: Unrecognized option or none given.\n";
 	print "usage: ViperDB [-init|-check|-checkstrict] [-md5]\n";
 	print "     -init	  Initializes the ViperDB Databases\n";
@@ -131,11 +131,13 @@ sub ParseConfigs
 				push @rignore, $_;
 			} elsif ($sub eq "dirs") {
 				$tmp = $_;
-				if ($tmp =~ /^recursive\s*(.+)/) {
-					$tmp = $1;
-					$rdirs{$tmp} = $tmp;
+				if ($tmp =~ s/^([a-zA-Z0-9,]+)\s+//) {
+					$opts = $1;
 				}
-				$cdirs{$tmp} = $tmp;
+				if ($opts =~ /recursive/) {
+					$rdirs{$tmp} = $opts;
+				}
+				$cdirs{$tmp} = $opts;
 			} elsif ( $sub eq "options") {
 				# add some sanity check pls.
 				if (/^hostname\s*=\s*(.+)/) {
@@ -157,6 +159,8 @@ sub ParseConfigs
 					$md5lock = $1;
 				} elsif (/^timelock\s*=\s*(.+)/) {
 					$timelock = $1;
+				} elsif (/^tielimit\s*=\s*(.+)/) {
+					$tielimit = $1;
 				} elsif (/^dbfname\s*=\s*(.+)/) {
 					$dbfname = $1;
 				} elsif (/^logfile\s*=\s*(.+)/) {
@@ -171,9 +175,8 @@ sub ParseConfigs
 	my $file;
 	while ($found) {
 		$found = 0;
-		foreach $dir (sort values %rdirs) {
-			if (!defined $dir) {
-				# undef->undef gets passed
+		foreach $dir (sort keys %rdirs) {
+			if (!defined $dir || ! -d $dir) {
 				# anyone know how to efficiently COMPLETELY delete contents of a hash
 				next;
 			}
@@ -195,8 +198,9 @@ sub ParseConfigs
 					$fname = "$dir/$file";
 					$fname =~ s/\/+/\//g;
 					if (-d $fname && !exists $cdirs{$fname} &&
-							! exists $ignore{"$file"}) {
-						$cdirs {$fname} = $ndirs{$fname} = $fname;
+						! exists $ignore{"$file"}) {
+						# pass options
+						$cdirs {$fname} = $ndirs{$fname} = $cdirs{$dir};
 						$found = 1;
 					}
 				}
@@ -269,21 +273,49 @@ sub OpenDB
 	# create or open.
 	$dbopened = 1;
 	if ($strict) {
-		if (! tie (%db, 'GDBM_File', $fname, &GDBM_READER, 0400)) {
-			$dbopened = 0;
-			&Err ("critical", 
-				"In $wd, can't tie $dbfname\nDatabase probably corrupt, suggesting deleting.");
+		$numtries = 0;
+		while (++$numtries <= $tielimit) {
+			untie %db;
+			tie (%db, 'GDBM_File', $fname, &GDBM_READER, 0400) && last;
+		}
+		if ($numtries != 1) {
+			if ($numtries >= $tielimit) {
+				$dbopened = 0;
+				&Err ("critical", 
+					"In $wd, can't tie $dbfname after $numtries tries.\nDatabase probably corrupt, suggesting deleting.");
+			} else {
+				&Err ("suspicious", 
+					"In $wd, tie to $dbfname needed $numtries tries.");
+			}
 		}
 	} else {
 		# if exists, chmod so we can write
 		( -f $fname ) && chmod (0600, $fname);
-		if (! tie (%db, 'GDBM_File', $fname, &GDBM_WRCREAT, 0600)) {
-			&Err ("strange", 
-				"In $wd, can't tie $dbfname\nDeleting and retrying...");
-			unlink $fname;
-			if (! tie (%db, 'GDBM_File', $fname, &GDBM_WRCREAT, 0600)) {
+		$numtries = 0;
+		while (++$numtries <= $tielimit) {
+			untie %db;
+			tie (%db, 'GDBM_File', $fname, &GDBM_WRCREAT, 0600) && last;
+		}
+		if ($numtries != 1) {
+			if ($numtries >= $tielimit) {
 				$dbopened = 0;
-				&Err ("critical", "Failed. Unable to monitor $wd.");
+				&Err ("critical", 
+					"In $wd, can't tie $dbfname after $numtries tries.\nDeleting and retrying...");
+				$numtries = 0;
+				unlink $fname;
+				while (++$numtries <= $tielimit) {
+					untie %db;
+					tie (%db, 'GDBM_File', $fname, &GDBM_WRCREAT, 0600) && last;
+				}
+				if ($numtries >= $tielimit) {
+					$dbopened = 0;
+					&Err ("critical", "Failed. Unable to monitor $wd.");
+				} else {
+					&Err ("note", "Success after $numtries tries.");
+				}
+			} else {
+				&Err ("suspicious", 
+					"In $wd, tie to $dbfname needed $numtries tries.");
 			}
 		}
 	}
@@ -322,7 +354,17 @@ sub GetAttr
 sub GetMD5
 {
 	my $file = shift;
-	if ( -f $file) {
+	if ( -l $file ) {
+		$digest = "symlink";
+	} elsif ( -p $file ) {
+		$digest = "fifo";
+	} elsif ( -S $file ) {
+		$digest = "socket";
+	} elsif ( -b $file ) {
+		$digest = "block device";
+	} elsif ( -d $file ) {
+		$digest = "character device";
+	} elsif ( -f $file) {
 		if ($domd5) {
 			open (MD5F, $file);
 			$md5obj->reset;
@@ -333,16 +375,6 @@ sub GetMD5
 		} else {
 			$digest = "file";
 		}
-	} elsif ( -l $file ) {
-		$digest = "symlink";
-	} elsif ( -p $file ) {
-		$digest = "fifo";
-	} elsif ( -S $file ) {
-		$digest = "socket";
-	} elsif ( -b $file ) {
-		$digest = "block device";
-	} elsif ( -d $file ) {
-		$digest = "character device";
 	} else {
 		$digest = "unknown";
 	}
@@ -368,6 +400,11 @@ sub ChkFile
 	if ( -d $file ) {
 		return;
 	}
+	# parse options
+	$nomtime = 0;
+	if ($cdirs{$wd} =~ /nomtime/) {
+		$nomtime = 1;
+	}
 	foreach $ig (@rignore) {
 		if ($file =~ /$ig/) {
 			if (!$strict) {
@@ -400,6 +437,9 @@ sub ChkFile
 	if ($digest eq "file" && &FType ($olddat[6]) eq "file") {
 		$digest = $olddat[6];
 	}
+	if ($nomtime) {
+		$mtime = "nomtime";
+	}
 	$newdat = "$size,$mode,$attr,$uid,$gid,$mtime,$digest";
 	if (! exists $db{"$file"}) {
 		&Err ("file added", "$fname");
@@ -497,7 +537,7 @@ sub ChkFile
 sub Go
 {
 	my $file;
-	foreach $wd (values %cdirs) {
+	foreach $wd (keys %cdirs) {
 		chdir $wd;
 		&OpenDB($wd);
 		if (!opendir(DIRLIST, $wd)) {
